import { Threads } from "@amodx/threads/";
//data
import { WorldRegister } from "../../../Data/World/WorldRegister.js";
import { RegionDataTool } from "../../../Default/Tools/Data/WorldData/RegionDataTool.js";
import { ColumnDataTool } from "../../../Default/Tools/Data/WorldData/ColumnDataTool.js";
import { ChunkDataTool } from "../../../Default/Tools/Data/WorldData/ChunkDataTool.js";
import { RegionHeaderRegister } from "../../../Data/RegionHeaderRegister.js";
import { DataLoaderTool } from "../../../Default/DataLoader/World/Tools/DataLoaderTool.js";
import { WorldSpaces } from "@divinevoxel/core/Data/World/WorldSpaces.js";
import { BuilderTool } from "../../../Default/Tools/Build/BuilderTool.js";
import { WorldLock } from "../Lock/WorldLock.js";
import { DVEFDataSync } from "../DVEFDataSync.js";
import { Column } from "../../../Data/World/Classes/Column.js";
export class WorldTasks {
    DVEW;
    columnTool = new ColumnDataTool();
    constructor(DVEW) {
        this.DVEW = DVEW;
    }
    getLocation(column) {
        return this.columnTool.setColumn(new Column(column)).getLocationData();
    }
    loadColumn(location, column) {
        WorldRegister.instance.column.add(location, column);
        DVEFDataSync.instance.worldData.column.sync(location);
    }
    unLoadColumn(data) {
        if (WorldLock.isLocked(data))
            return false;
        this.DVEW.dataSync.worldData.column.unSync(data);
        WorldRegister.instance.column.remove(data);
        const region = WorldRegister.instance.region.get(data);
        if (region && region.columns.size == 0) {
            WorldRegister.instance.region.remove(data);
            this.DVEW.dataSync.worldData.region.unSync(data);
            return true;
        }
        return false;
    }
    loadRegionHeader(location, data) {
        RegionHeaderRegister.add(location, data);
        DVEFDataSync.instance.worldData.regionHeader.sync(location);
    }
}
export default function (DVEW) {
    const regionTool = new RegionDataTool();
    const columnTool = new ColumnDataTool();
    const chunkTool = new ChunkDataTool();
    const dataLoaderTool = new DataLoaderTool();
    const builderTool = new BuilderTool();
    const loadInMap = new Map();
    Threads.registerTasks("add-chunk", async (location) => {
        const chunk = WorldRegister.instance.chunk.get(location);
        if (chunk) {
            DVEW.dataSync.worldData.chunk.sync(location);
            return;
        }
        if (dataLoaderTool.isEnabled()) {
            WorldSpaces.column.getPositionLocation(location);
            const columnLocation = [
                ...WorldSpaces.column.getLocation(),
            ];
            if (loadInMap.has(columnLocation.toString()))
                return;
            loadInMap.set(columnLocation.toString(), true);
            const success = await dataLoaderTool.loadIfExists(columnLocation);
            loadInMap.delete(columnLocation.toString());
            if (!success) {
                builderTool.setLocation(columnLocation).fillColumn();
            }
            return;
        }
        if (!chunk) {
            WorldRegister.instance.column.fill(location);
        }
    });
    Threads.registerTasks("world-alloc", async (data, onDone) => {
        await WorldLock.addLock(data);
        if (onDone)
            onDone();
    }, "deferred");
    Threads.registerTasks("world-dealloc", async (data, onDone) => {
        await WorldLock.removeLock(data);
        if (onDone)
            onDone();
    }, "deferred");
    Threads.registerTasks("unload-column", (data, onDone) => {
        const resutls = DVEW.tasks.unLoadColumn(data);
        return onDone ? onDone(resutls) : resutls;
    }, "deferred");
    Threads.registerTasks("load-region-header", ([location, data], onDone) => {
        const resutls = DVEW.tasks.loadRegionHeader(location, data);
        return onDone ? onDone(resutls) : resutls;
    });
    Threads.registerTasks("load-column", ([location, column], onDone) => {
        const resutls = DVEW.tasks.loadColumn(location, column);
        return onDone ? onDone(resutls) : resutls;
    });
    Threads.registerTasks("clear-all", () => {
        WorldRegister.instance.clearAll();
    });
}
