import { LocationData } from "@divinevoxel/core/Math/index.js";
import { VoxelConstructors } from "./Constructors/Voxel/VoxelConstructors.js";
import { ChunkProcessor } from "./Processors/ChunkProcessor.js";
import { OverrideManager } from "./Rules/Overrides/OverridesManager.js";
import { DVEBuilder } from "../../Interfaces/Builder/DVEBuilder.js";
import { BuildNodeMesh } from "./Tasks/BuidlerTasks.types.js";
import { VoxelConstructor } from "./Constructors/Voxel/Classes/VoxelConstructor.js";
import { VoxelShapeManager } from "./Shapes/VoxelShapeManager.js";
export type DVEDefaultBuilderInitData = {
    constructors: VoxelConstructor[];
};
export declare class DVEDefaultBuilder extends DVEBuilder {
    static instance: DVEDefaultBuilder;
    static get defaults(): {
        box: {
            shapeState(id: string, textures: import("../../Textures/Constructor.types.js").ConstructorTextureData[] | Record<import("@divinevoxel/core/Math/index.js").VoxelFaces, import("../../Textures/Constructor.types.js").ConstructorTextureData[]>): import("./Constructors/Voxel/Classes/Box/ShapeStateBox.constructor.js").ShapeStateBoxVoxelConstructor;
            simple(id: string, textures: import("../../Textures/Constructor.types.js").ConstructorTextureData | Record<import("@divinevoxel/core").DirectionNames, import("../../Textures/Constructor.types.js").ConstructorTextureData>): import("./Constructors/Voxel/Classes/Box/SimpleBox.constructor.js").SimpleBoxVoxelConstructor;
            pillar(id: string, textures: import("./Constructors/Voxel/Classes/Box/PillarBox.constructor.js").PillarBoxVoxelConstructorData): import("./Constructors/Voxel/Classes/Box/PillarBox.constructor.js").PillarBoxVoxelConstructor;
        };
        stair: {
            simple(id: string, texture: import("../../Textures/Constructor.types.js").ConstructorTextureData): import("./Constructors/Voxel/Classes/Stair/SimpleStair.constructor.js").SimpleStairVoxelConstructor;
        };
        panel: {
            simple(id: string, texture: import("../../Textures/Constructor.types.js").ConstructorTextureData): import("./Constructors/Voxel/Classes/Panel/SimplePanel.constructor.js").SimplePanelVoxelConstructor;
        };
        crossedPanel: {
            simple(id: string, texture: import("../../Textures/Constructor.types.js").ConstructorTextureData): import("./Constructors/Voxel/Classes/Panel/SimpleCrossedPanel.constructor.js").SimpleCrossedPanelVoxelConstructor;
        };
        liquid: {
            simple(id: string, textures: [import("../../Textures/Constructor.types.js").ConstructorTextureData, import("../../Textures/Constructor.types.js").ConstructorTextureData]): import("./Constructors/Voxel/Classes/Liquid/SimpleLiquid.constructor.js").SimpleLiquidConstructor;
        };
    };
    constructors: typeof VoxelConstructors;
    textureManager: {
        textureDataHasBeenSet: boolean;
        data: import("../../Textures/Texture.types.js").TextureTypeUVMap;
        getTextureUV(data: import("../../Textures/Constructor.types.js").ConstructorTextureData, overlay?: boolean): number;
        setTextureIndex(data: import("../../Textures/Texture.types.js").TextureTypeUVMap): void;
        releaseTextureData(): void;
        isReady(): boolean;
    };
    chunkProcessor: ChunkProcessor;
    nodes: {
        builders: Map<string, import("./Meshers/Classes/Mesher.js").Mesher>;
        registerBuilder(builder: import("./Meshers/Classes/Mesher.js").Mesher): void;
        buildNode(data: BuildNodeMesh): [import("./Tasks/BuidlerTasks.types.js").SetNodeMesh, ArrayBuffer[]] | false;
    };
    overrides: typeof OverrideManager;
    renderedSubstances: {
        meshers: import("../../Util/UtilMap.js").UtilMap<string, import("./Tools/VoxelMesherDataTool.js").VoxelMesherDataTool>;
        add(id: string): void;
    };
    shapes: typeof VoxelShapeManager;
    observers: {
        texturesRegistered: import("@amodx/core/Observers/Observable.js").Observable<{
            textureDataHasBeenSet: boolean;
            data: import("../../Textures/Texture.types.js").TextureTypeUVMap;
            getTextureUV(data: import("../../Textures/Constructor.types.js").ConstructorTextureData, overlay?: boolean): number;
            setTextureIndex(data: import("../../Textures/Texture.types.js").TextureTypeUVMap): void;
            releaseTextureData(): void;
            isReady(): boolean;
        }>;
    };
    constructor(data: DVEDefaultBuilderInitData);
    init(): void;
    buildChunk(location: LocationData, LOD?: number, priority?: number): boolean;
}
