import { WorldSpaces } from "@divinevoxel/core/Data/World/WorldSpaces";
import { Compressor } from "@amodx/core/Compression/Compression";
export class WorldDataTool {
    worldDataBase;
    async setPath(id) {
        return true;
    }
    constructor(worldDataBase) {
        this.worldDataBase = worldDataBase;
    }
    dataType = "world-data";
    setType(data) {
        this.dataType = data;
    }
    async columnExists(location) {
        await this.worldDataBase.setDimension(location[0]);
        return (await this.columnTimestamp(location)) > 0;
    }
    async columnExistsBatch(locations) {
        await this.worldDataBase.setDimension(locations[0][0]);
        const existsRecord = {};
        const transication = await this.worldDataBase.database.getTransaction("readonly", this.worldDataBase.typeStores["world-data"].id);
        const proms = [];
        const keys = [];
        for (const location of locations) {
            const key = location.toString();
            keys.push(key);
            proms.push(transication.get(this.worldDataBase.columnTimestamp.getKey(WorldSpaces.column.getKeyLocation(location), this.dataType)));
        }
        const resolved = await Promise.all(proms);
        for (let i = 0; i < keys.length; i++) {
            existsRecord[keys[i]] = resolved[i] !== undefined;
        }
        return existsRecord;
    }
    async columnTimestamp(location) {
        await this.worldDataBase.setDimension(location[0]);
        const stamp = await this.worldDataBase.columnTimestamp.get(WorldSpaces.column.getKeyLocation(location), this.dataType);
        if (!stamp)
            return 0;
        return stamp;
    }
    async loadRegionHeader(location) {
        await this.worldDataBase.setDimension(location[0]);
        const buffer = await this.worldDataBase.regionHeader.get(WorldSpaces.region.getKeyLocation(location), this.dataType);
        if (!buffer)
            return new ArrayBuffer(0);
        return buffer;
    }
    async loadColumn(location) {
        await this.worldDataBase.setDimension(location[0]);
        const buffer = await this.worldDataBase.column.get(WorldSpaces.column.getKeyLocation(location), this.dataType);
        if (!buffer)
            return new ArrayBuffer(0);
        return await (await Compressor.decompressArray(buffer, "Uint8")).buffer;
    }
    async saveColumn(location, buffer) {
        const compressed = await Compressor.compressArray(new Uint8Array(buffer));
        await this.worldDataBase.setDimension(location[0]);
        await this.worldDataBase.column.set(WorldSpaces.column.getKeyLocation(location), this.dataType, compressed.buffer);
        await this.worldDataBase.columnTimestamp.set(WorldSpaces.column.getKeyLocation(location), this.dataType, Date.now());
        return true;
    }
}
