import { ColumnDataTool } from "../../../Tools/Data/WorldData/ColumnDataTool.js";
import { WorldRegister } from "../../../../Data/World/WorldRegister.js";
import { Distance3D } from "@divinevoxel/core/Math/Functions/Distance3d.js";
import { EngineSettings } from "@divinevoxel/core/Data/Settings/EngineSettings.js";
import { DataHooks } from "../../../../Data/DataHooks.js";
import { WorldLock } from "../../../../Contexts/World/Lock/WorldLock.js";
import { DataHanlderWrapper } from "../DataHandlerWrapper.js";
export class DataLoaderTool {
    static columnDataTool = new ColumnDataTool();
    dimension = "main";
    mode = "server";
    _enabled = typeof DataHanlderWrapper.instance !== "undefined";
    constructor() {
        this.mode = EngineSettings.settings.data.mode;
        DataHooks.settingsSynced.subscribe("data-loader", (data) => {
            this.mode = data.data.mode;
        });
    }
    setDimension(dimension) {
        this.dimension = dimension;
    }
    isEnabled() {
        return !(typeof DataHanlderWrapper.instance?.columnExists === "undefined");
    }
    async saveColumnIfNotStored(location) {
        if (!DataLoaderTool.columnDataTool.setLocation(location).loadIn())
            return false;
        if (DataLoaderTool.columnDataTool.isStored())
            return false;
        await this.saveColumn(location);
        return true;
    }
    async loadIfExists(location) {
        const exists = await this.columnExists(location);
        if (exists) {
            await this.loadColumn(location);
        }
        return false;
    }
    async saveColumn(location) {
        await DataHanlderWrapper.instance.saveColumn(location);
    }
    async loadColumn(location) {
        await DataHanlderWrapper.instance.loadColumn(location);
    }
    async unLoadColumn(location) {
        if (WorldLock.isLocked(location) ||
            !WorldRegister.instance.column.get(location))
            return false;
        await DataHanlderWrapper.instance.unLoadColumn(location);
    }
    async columnExists(location) {
        return await DataHanlderWrapper.instance.columnExists(location);
    }
    async columnExistsBatch(location) {
        return await DataHanlderWrapper.instance.columnExistsBatch(location);
    }
    async columnTimestamp(location) {
        return await DataHanlderWrapper.instance.columnTimestamp(location);
    }
    async unLoadAllOutsideRadius([, sx, , sz], radius, run = (columntool) => true) {
        const proms = [];
        this.allColumns((column) => {
            const location = column.getLocationData();
            const distnace = Distance3D(location[1], 0, location[3], sx, 0, sz);
            if (distnace > radius) {
                if (!run(column))
                    return;
                proms.push(this.unLoadColumn([...column.getLocationData()]));
            }
        });
        await Promise.all(proms);
    }
    async unLoadAllColumns(onDone) {
        const proms = [];
        this.allColumns((column) => {
            proms.push(this.unLoadColumn([...column.getLocationData()]));
        });
        await Promise.all(proms);
    }
    allColumns(run) {
        const dim = WorldRegister.instance.dimensions.get(this.dimension);
        if (!dim)
            return;
        for (const [key, region] of dim.regions) {
            for (const column of region.getColumns()) {
                DataLoaderTool.columnDataTool.setColumn(column);
                run(DataLoaderTool.columnDataTool);
            }
        }
    }
    async allColumnsAsync(run) {
        const dim = WorldRegister.instance.dimensions.get(this.dimension);
        if (!dim)
            return;
        for (const [key, region] of dim.regions) {
            for (const column of region.getColumns()) {
                DataLoaderTool.columnDataTool.setColumn(column);
                await run(DataLoaderTool.columnDataTool);
            }
        }
    }
    getAllUnStoredColumns(run) {
        this.allColumns((column) => {
            if (!column.isStored()) {
                run(...column.getLocationData());
            }
        });
    }
}
