import { Generator } from "./Classes/Generator.js";
import { RegisterDefaultTasks } from "./Tasks/RegisterDefaultTasks.js";
import { WorldSpaces } from "@divinevoxel/core/Data/World/WorldSpaces.js";
import { IWGInitalLoad } from "./Load/InitLoad.js";
import { IWGSafeExit } from "./Load/SafeExit.js";
import { IWGTeleport } from "./Load/Teleport.js";
export class IWG {
    static inProgressMap = {
        map: new Map(),
        add(x, y, z, tasks) {
            this.map.set(IWG.getKey(x, y, z), tasks);
        },
        has(x, y, z) {
            return this.map.has(IWG.getKey(x, y, z));
        },
        remove(x, y, z) {
            return this.map.delete(IWG.getKey(x, y, z));
        },
    };
    static getKey(x, y, z) {
        return WorldSpaces.column.getKeyXYZ(x, y, z);
    }
    static generators = new Map();
    static addGenerator(data) {
        const gen = new Generator(data);
        this.generators.set(gen.data.id, gen);
        return gen;
    }
    static getGenerator(id) {
        const gen = this.generators.get(id);
        if (!gen)
            return false;
        return gen;
    }
    static removeGenerator(id) {
        return this.generators.delete(id);
    }
    static worldGenUpdate(max = 5) {
        for (const [key, gen] of this.generators) {
            gen.worldGenUpdate(max);
        }
    }
    static searchUpdate() {
        for (const [key, gen] of this.generators) {
            gen.generateUpdate();
        }
    }
    static saveUpdate(max = 5) {
        for (const [key, gen] of this.generators) {
            gen.saveUpdate(max);
        }
    }
    static async initalLoad(dimension, position, settings = {
        renderDistance: 100,
        generateDistance: 150,
    }, onCheck = (gen) => { }) {
        const tempGen = this.addGenerator({
            id: "init-gen",
            positionWatch: position,
            ...settings,
        });
        tempGen.updateDimension(dimension);
        const loader = new IWGInitalLoad(tempGen);
        await loader.run(onCheck);
        this.removeGenerator(tempGen.data.id);
    }
    static async safeExit(genId, onCheck = (gen) => { }) {
        const gen = this.getGenerator(genId);
        if (!gen)
            return false;
        const loader = new IWGSafeExit(gen);
        await loader.run(onCheck);
        this.removeGenerator(gen.data.id);
    }
    static async teleport(genId, location, onCheck = (gen) => { }) {
        const gen = this.getGenerator(genId);
        if (!gen)
            return false;
        const loader = new IWGTeleport(gen, location);
        await loader.run(onCheck);
    }
}
RegisterDefaultTasks();
