import { Flat3DIndex, Traverse } from "@amodx/math";
import { StringPalette } from "@divinevoxel/core/Interfaces/Data/StringPalette";
import { NumberPalette } from "@divinevoxel/core/Interfaces/Data/NumberPalette";
import { VoxelPaletteReader } from "@divinevoxel/core/Data/Voxel/VoxelPalette";
export class VoxelTemplate {
    data;
    index = Flat3DIndex.GetXZYOrder();
    palette = new StringPalette();
    statePalette = new NumberPalette();
    constructor(data) {
        this.data = data;
        this.index.setBounds(...this.data.size);
        data.palette.forEach((_) => this.palette.register(_));
    }
    *traverse() {
        const end = this.data.size;
        const raw = [0, 0, 0, 0];
        const curosr = {
            position: [0, 0, 0],
            raw,
        };
        for (const { x, y, z } of Traverse.FromToVec3([0, 0, 0], [end[0] - 1, end[1] - 1, end[2] - 1], 1)) {
            const vindex = this.index.getIndexXYZ(x, y, z);
            curosr.position[0] = x;
            curosr.position[1] = y;
            curosr.position[2] = z;
            raw[0] = VoxelPaletteReader.id.numberFromString(this.palette.getStringId(this.data.voxels.ids[vindex]));
            raw[1] = 0;
            raw[2] = this.statePalette.getValue(this.data.voxels.state[vindex]);
            raw[3] = VoxelPaletteReader.id.numberFromString(this.palette.getStringId(this.data.voxels.secondary[vindex]));
            if (raw[0] < 1 && raw[3] < 1)
                continue;
            yield curosr;
        }
    }
    toJSON() {
        return {
            size: this.data.size,
            palette: this.data.palette,
            statePalette: this.data.statePalette,
            voxels: this.data.voxels,
        };
    }
}
