;
import { DimensionsRegister } from "../../../Data/World/DimensionsRegister.js";
import { LocationBoundTool } from "./LocationBoundTool.js";
import { WorldDataStructProperties } from "../../../Data/Constants/Structs/WorldDataStructProperties.js";
import { arrayBufferToSharedArrayBuffer } from "@amodx/core/Buffers/arrayBufferToSharedArrayBuffer.js";
export class DataToolBase extends LocationBoundTool {
    struct;
    _c;
    _dimensionRegister = new DimensionsRegister();
    constructor() {
        super();
    }
    getStructValue(id) {
        this.struct.setBuffer(this._c);
        return this.struct.getProperty(id);
    }
    setTagValue(id, value) {
        this.struct.setBuffer(this._c);
        return this.struct.setProperty(id, value);
    }
    getArrayTagValue(id, index) {
        this.struct.setBuffer(this._c);
        return this.struct.getArrayPropertyValue(id, index);
    }
    setArrayTagValue(id, index, value) {
        this.struct.setBuffer(this._c);
        return this.struct.setArrayPropertyValue(id, index, value);
    }
    setBuffer(buffer) {
        this._c = buffer;
        this.struct.setBuffer(this._c);
    }
    getBuffer() {
        if (this._c instanceof DataView)
            return this._c.buffer;
        return this._c;
    }
    getAsArrayBuffer() {
        const buffer = this.getBuffer();
        if (buffer instanceof ArrayBuffer)
            return buffer;
        return arrayBufferToSharedArrayBuffer(this.getBuffer());
    }
    getBufferSize() {
        return this.struct.getBuffer().byteLength;
    }
    loadInAt(x, y, z) {
        this.setXYZ(x, y, z);
        return this.loadIn();
    }
    loadInVec3Array(vec3) {
        this.setXYZ(vec3[0], vec3[1], vec3[2]);
        return this.loadIn();
    }
    loadInVec3(vec3) {
        this.setXYZ(vec3.x, vec3.y, vec3.z);
        return this.loadIn();
    }
    loadInAtLocation(location) {
        this.setLocation(location);
        return this.loadIn();
    }
}
export class EncodedPositionDataTool extends DataToolBase {
    position = { x: 0, y: 0, z: 0 };
    constructor() {
        super();
    }
    getPositionData() {
        this.position.x = this.getStructValue(WorldDataStructProperties.positionX);
        this.position.y = this.getStructValue(WorldDataStructProperties.positionY);
        this.position.z = this.getStructValue(WorldDataStructProperties.positionZ);
        return this.position;
    }
    setPositionData(x, y, z) {
        this.setTagValue(WorldDataStructProperties.positionX, x);
        this.setTagValue(WorldDataStructProperties.positionY, y);
        this.setTagValue(WorldDataStructProperties.positionZ, z);
        return this.position;
    }
    setDimensionId(dimensionId) {
        this.setTagValue(WorldDataStructProperties.dimensionId, this._dimensionRegister.getDimensionNumericId(dimensionId));
    }
    getDimensionId() {
        return this._dimensionRegister.getDimensionStringId(this.getStructValue(WorldDataStructProperties.dimensionId));
    }
    getLocationData() {
        const pos = this.getPositionData();
        return [this.getDimensionId(), pos.x, pos.y, pos.z];
    }
}
