import type { RawVoxelData } from "@divinevoxel/core/Types/Voxel.types.js";
import { VoxelStruct } from "@divinevoxel/core/Data/Voxel/VoxelStruct.js";
import { ChunkDataTool } from "./WorldData/ChunkDataTool.js";
import { HeightMapTool } from "./WorldData/HeightMapTool.js";
import { DataToolBase } from "../Classes/DataToolBase.js";
import { ColumnDataTool } from "./WorldData/ColumnDataTool.js";
import { SubstanceDataTool } from "./SubstanceDataTool.js";
import { LocationData } from "@divinevoxel/core/Math/index.js";
import { AddVoxelData } from "Data/Types/WorldData.types.js";
export declare enum DataToolModes {
    /**# World Data Mode
     * ---
     * Read data directly from the world.
     */
    WORLD = 0,
    /**# Voxel Matrix Mode
     * ---
     * Read from a voxel matrix.
     */
    VOXEL_MATRIX = 1,
    /**# Voxel Data Mode
     * ---
     * Read data from a single voxel passed in via `loadInRaw`
     */
    VOXEL_DATA = 2
}
export declare class DataTool extends DataToolBase {
    static GetVoxelIDFromString(id: string): number | undefined;
    static GetVoxelIDFromNumber(id: number): string;
    static IsSameVoxel(dataTool1: DataTool, dataTool2: DataTool): boolean;
    static VoxelDataToRaw(data: Partial<AddVoxelData>, light?: number): RawVoxelData;
    static Modes: typeof DataToolModes;
    static _heightMapTool: HeightMapTool;
    static _columntool: ColumnDataTool;
    _chunkTool: ChunkDataTool;
    _substanceTool: SubstanceDataTool;
    _loadedIn: boolean;
    _mode: DataToolModes;
    data: {
        raw: RawVoxelData;
        id: number;
        secondaryId: number;
    };
    /**## secondary
     * If the data tool is processing secondary voxoels
     */
    __secondary: boolean;
    private _loadedId;
    constructor();
    setMode(mode: DataToolModes): this;
    clear(): this;
    setDimension(dimensionId: string | number): this;
    setSecondary(enable: boolean): this;
    _getBaseId(id: number): number;
    getSubstnaceData(): SubstanceDataTool;
    getRaw(): RawVoxelData;
    waitTillCanLoad(locatin: LocationData, maxWaitTime?: number): Promise<boolean>;
    loadInRaw(rawData: RawVoxelData): this;
    loadInData(data: Partial<AddVoxelData>, light?: number): void;
    __struct: VoxelStruct;
    private __process;
    loadIn(): boolean;
    commit(heightMapUpdate?: number): boolean;
    hasRGBLight(): boolean;
    hasSunLight(): boolean;
    getLight(): number;
    setLight(light: number): this;
    isOpaque(): boolean;
    getLevel(): number;
    setLevel(level: number): this;
    getLevelState(): number;
    setLevelState(state: number): this;
    getShapeState(): number;
    setShapeState(state: number): this;
    hasSecondaryVoxel(): boolean;
    getShapeId(): number;
    getShapeStringId(): string | null;
    isLightSource(): boolean;
    getLightSourceValue(): number;
    getSubstanceStringId(): string;
    getSubstance(): number;
    getMaterial(): string | null;
    getHardness(): number;
    getCollider(): string | null;
    checkCollisions(): boolean;
    isRich(): boolean | 0;
    setAir(): this;
    isAir(): boolean;
    setBarrier(): this;
    isBarrier(): boolean;
    getId(base?: boolean): number;
    setId(id: number): this;
    setStringId(id: string): this;
    getStringId(): string;
    setName(name: string): void;
    getName(): string;
    isRenderable(): boolean;
    isSameVoxel(dataTool: DataTool): boolean;
}
