import { Threads } from "@amodx/threads/";
import { WorldRegister } from "../../../Data/World/WorldRegister.js";
import { WorldSpaces } from "@divinevoxel/core/Data/World/WorldSpaces.js";
import { ConstructorRemoteThreadTasks } from "../../../Contexts/Common/ConstructorRemoteThreadTasks.js";
import { DVEFWorldCore } from "../../../Contexts/World/DVEFWorldCore.js";
import { ConstructorTasksIds } from "../../../Contexts/Common/ConstructorTasksIds.js";
export class TaskTool {
    _data = {
        dimension: "main",
        queue: "main",
    };
    _thread = "";
    _priority = 0;
    constructor() {
        this._thread = Threads.threadName;
    }
    setPriority(priority) {
        this._priority = priority;
        return this;
    }
    setFocalPoint(location) {
        const [dimesnion, x, y, z] = location;
        const queueKey = `${dimesnion}-${WorldSpaces.region.getKeyXYZ(x, y, z)}`;
        DVEFWorldCore.instance.queues.addQueue(queueKey);
        this._data.queue = queueKey;
        this._thread = Threads.threadName;
        return this;
    }
    voxelUpdate = {
        update: {
            run: (location, raw, onDone, mode = "sync") => {
                DVEFWorldCore.instance.threads.constructors.runPromiseTasks(ConstructorTasksIds.VoxelUpdate, [location, raw, this._data.queue, this._thread], [], onDone, mode == "sync" ? 0 : undefined);
            },
        },
        erase: {
            run: (location, onDone, mode = "sync") => {
                DVEFWorldCore.instance.threads.constructors.runPromiseTasks(ConstructorTasksIds.VoxelErease, [location, this._data.queue, this._thread], [], onDone, mode == "sync" ? 0 : undefined);
            },
        },
        paint: {
            run: (location, raw, onDone, mode = "sync") => {
                DVEFWorldCore.instance.threads.constructors.runPromiseTasks(ConstructorTasksIds.VoxelPaint, [location, raw, this._data.queue, this._thread], [], onDone, mode == "sync" ? 0 : undefined);
            },
        },
    };
    build = {
        chunk: {
            deferred: {
                run: (buildTasks, onDone) => {
                    DVEFWorldCore.instance.threads.constructors.runPromiseTasks(ConstructorTasksIds.BuildChunk, {
                        data: buildTasks,
                        priority: this._priority,
                    }, [], onDone, undefined, 0);
                },
            },
            queued: {
                add: (location) => {
                    DVEFWorldCore.instance.queues.buildChunk.add({
                        data: [location, 1],
                        priority: this._priority,
                    }, this._data.queue);
                },
                run: (onDone) => {
                    DVEFWorldCore.instance.queues.buildChunk.run(this._data.queue);
                    DVEFWorldCore.instance.queues.buildChunk.onDone(this._data.queue, onDone);
                },
                runAndAwait: async () => {
                    await DVEFWorldCore.instance.queues.buildChunk.runAndAwait(this._data.queue);
                },
            },
        },
        column: {
            queued: {},
            deferred: {
                run: (location, onDone) => {
                    DVEFWorldCore.instance.threads.constructors.runPromiseTasks(ConstructorTasksIds.BuildColumn, [location, 1], [], onDone, undefined, 0);
                },
            },
        },
    };
    explosion = {
        run: (location, radius, onDone) => {
            DVEFWorldCore.instance.threads.constructors.runPromiseTasks(ConstructorTasksIds.Explosion, [location, radius, "", ""], [], onDone, undefined, 0);
        },
    };
    anaylzer = {
        update: {
            run: (location, onDone) => {
                DVEFWorldCore.instance.threads.constructors.runPromiseTasks(ConstructorTasksIds.AnalyzerUpdate, [location, this._data.queue, this._thread], [], onDone, undefined, 0);
            },
        },
    };
    propagation = {
        deferred: {
            run: (location, onDone) => {
                DVEFWorldCore.instance.threads.constructors.runPromiseTasks(ConstructorTasksIds.AnalyzerPropagation, [location, this._data.queue, this._thread], [], onDone, undefined, 0);
            },
        },
        queued: {
            add: (location) => {
                DVEFWorldCore.instance.queues.propagation.add([location, this._data.queue, this._thread], this._data.queue);
            },
            run: (onDone) => {
                DVEFWorldCore.instance.queues.propagation.run(this._data.queue);
                DVEFWorldCore.instance.queues.propagation.onDone(this._data.queue, onDone);
            },
            runAndAwait: async () => {
                await DVEFWorldCore.instance.queues.propagation.runAndAwait(this._data.queue);
            },
        },
    };
    generate = {
        deferred: {
            run(location, data, onDone) {
                DVEFWorldCore.instance.threads.constructors.runPromiseTasks(ConstructorTasksIds.Generate, [location, data], [], onDone, undefined, 0);
            },
        },
        queued: {
            add: (data) => {
                DVEFWorldCore.instance.queues.generate.add(data, this._data.queue);
            },
            run: (onDone) => {
                DVEFWorldCore.instance.queues.generate.run(this._data.queue);
                DVEFWorldCore.instance.queues.generate.onDone(this._data.queue, onDone);
            },
            runAndAwait: async () => {
                await DVEFWorldCore.instance.queues.generate.runAndAwait(this._data.queue);
            },
        },
    };
    decorate = {
        deferred: {
            run: (location, data, onDone) => {
                DVEFWorldCore.instance.threads.constructors.runPromiseTasks(ConstructorTasksIds.Decorate, [location, data], [], onDone, undefined, 0);
            },
        },
        queued: {
            add: async (data) => {
                DVEFWorldCore.instance.queues.decorate.add(data, this._data.queue);
            },
            run: (onDone) => {
                DVEFWorldCore.instance.queues.decorate.run(this._data.queue);
                DVEFWorldCore.instance.queues.decorate.onDone(this._data.queue, onDone);
            },
            runAndAwait: async () => {
                await DVEFWorldCore.instance.queues.decorate.runAndAwait(this._data.queue);
            },
        },
    };
    worldSun = {
        deferred: {
            run: (location, onDone) => {
                DVEFWorldCore.instance.threads.constructors.runPromiseTasks(ConstructorTasksIds.WorldSun, [location, this._thread], [], onDone, undefined, 0);
            },
        },
        queued: {
            add: (location) => {
                DVEFWorldCore.instance.queues.worldSun.add([location, this._data.queue, this._thread], this._data.queue);
                WorldRegister.instance.column.fill(location);
            },
            run: (onDone) => {
                DVEFWorldCore.instance.queues.worldSun.run(this._data.queue);
                DVEFWorldCore.instance.queues.worldSun.onDone(this._data.queue, onDone);
            },
            runAndAwait: async () => {
                await DVEFWorldCore.instance.queues.worldSun.runAndAwait(this._data.queue);
            },
        },
    };
}
const tasks = new TaskTool();
const CommonTasks = {
    buildChunk: Threads.registerTasks(ConstructorRemoteThreadTasks.buildChunk, (data) => {
        tasks.setPriority(data.priority);
        tasks.build.chunk.deferred.run(data.data, () => { });
    }),
};
