//types
//objects
import { WorldBounds } from "@divinevoxel/core/Data/World/WorldBounds.js";
import { WorldGenRegister } from "./WorldGenRegister.js";
//tools
import { WorldGenBrush } from "./WorldGenBrush.js";
import { SafeInterval } from "@amodx/core/Intervals/SafeInterval.js";
import { Threads } from "@amodx/threads";
import { ConstructorTasksIds } from "../../Contexts/Common/ConstructorTasksIds.js";
export class WorldGeneration {
    static worldGen = null;
    static register = WorldGenRegister;
    static worldBounds = WorldBounds;
    static _brushes = [];
    static setWorldGen(worldGen) {
        this.worldGen = worldGen;
    }
    static async generate(data, mode, onDone) {
        if (!this.worldGen) {
            throw new Error(`A World Generator must be set.`);
        }
        const requestsId = WorldGenRegister.registerRequest(data[0]);
        for (const brush of this._brushes) {
            brush.requestsId = requestsId;
        }
        if (mode == "generate") {
            await this.worldGen.generate(data);
        }
        if (mode == "decorate") {
            await this.worldGen.decorate(data);
        }
        const inte = new SafeInterval().setInterval(100).setOnRun(() => {
            if (WorldGenRegister.attemptRequestFullFill(requestsId)) {
                onDone();
                inte.stop();
            }
        });
        inte.start();
    }
    static getBrush() {
        const brush = new WorldGenBrush();
        return brush;
    }
}
Threads.registerTasks(ConstructorTasksIds.Generate, (data, onDone) => {
    if (!onDone)
        return;
    WorldGeneration.generate(data, "generate", onDone);
}, "deferred");
Threads.registerTasks(ConstructorTasksIds.Decorate, (data, onDone) => {
    if (!onDone)
        return;
    WorldGeneration.generate(data, "decorate", onDone);
}, "deferred");
