export class DataMatrix {
    size;
    defaultValue;
    _matrix;
    constructor(size, defaultValue) {
        this.size = size;
        this.defaultValue = defaultValue;
        this._matrix = [];
        this.setAll(defaultValue);
    }
    get sizeX() {
        return this.size[0];
    }
    get sizeY() {
        return this.size[1];
    }
    get sizeZ() {
        return this.size[2];
    }
    setMatrix(matrix) {
        this._matrix = matrix;
    }
    cloneMatrix() {
        return structuredClone(this._matrix);
    }
    setVec3([x, y, z], value) {
        if (!this.isWithinBounds(x, y, z))
            return;
        if (!this._matrix[x])
            this._matrix[x] = [];
        if (!this._matrix[x][y])
            this._matrix[x][y] = [];
        this._matrix[x][y][z] = value;
    }
    setXYZ(x, y, z, value) {
        this.setVec3([x, y, z], value);
    }
    getVec3([x, y, z]) {
        return this.isWithinBounds(x, y, z) ? this._matrix[x]?.[y]?.[z] : undefined;
    }
    getXYZ(x, y, z) {
        return this.getVec3([x, y, z]);
    }
    deleteVec3([x, y, z]) {
        if (this.isWithinBounds(x, y, z))
            this._matrix[x][y][z] = this.defaultValue;
    }
    deleteXYZ(x, y, z) {
        this.deleteVec3([x, y, z]);
    }
    setAll(value) {
        for (let x = 0; x < this.size[0]; x++) {
            this._matrix[x] = [];
            for (let y = 0; y < this.size[1]; y++) {
                this._matrix[x][y] = [];
                for (let z = 0; z < this.size[2]; z++) {
                    this._matrix[x][y][z] = value;
                }
            }
        }
    }
    *getAll() {
        for (let x = 0; x < this.size[0]; x++) {
            for (let y = 0; y < this.size[1]; y++) {
                for (let z = 0; z < this.size[2]; z++) {
                    yield [[x, y, z], this._matrix[x][y][z]];
                }
            }
        }
    }
    swap(startPosition, endPosition) {
        const startValue = this.getVec3(startPosition);
        const endValue = this.getVec3(endPosition);
        if (startValue !== undefined && endValue !== undefined) {
            this.setVec3(startPosition, endValue);
            this.setVec3(endPosition, startValue);
        }
    }
    isCube() {
        return this.sizeX === this.sizeY && this.sizeZ === this.sizeX;
    }
    isWithinBounds(x, y, z) {
        return (x >= 0 &&
            x < this.size[0] &&
            y >= 0 &&
            y < this.size[1] &&
            z >= 0 &&
            z < this.size[2]);
    }
}
